@@ -1,5 +1,6 @@
 #pragma once
 
+#include "../dxvk/dxvk_cs.h"
 #include "../dxvk/dxvk_device.h"
 
 #include "../d3d10/d3d10_buffer.h"
@@ -67,9 +68,7 @@ namespace dxvk {
     }
 
     D3D11_COMMON_BUFFER_MAP_MODE GetMapMode() const {
-      return (m_buffer->memFlags() & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)
-        ? D3D11_COMMON_BUFFER_MAP_MODE_DIRECT
-        : D3D11_COMMON_BUFFER_MAP_MODE_NONE;
+      return m_mapMode;
     }
 
     Rc<DxvkBuffer> GetBuffer() const {
@@ -77,15 +76,23 @@ namespace dxvk {
     }
     
     DxvkBufferSlice GetBufferSlice() const {
-      return GetBufferSlice(0, m_desc.ByteWidth);
+      return DxvkBufferSlice(m_buffer, 0, m_desc.ByteWidth);
     }
     
     DxvkBufferSlice GetBufferSlice(VkDeviceSize offset) const {
-      return GetBufferSlice(offset, m_desc.ByteWidth - offset);
+      VkDeviceSize size = m_desc.ByteWidth;
+
+      return likely(offset < size)
+        ? DxvkBufferSlice(m_buffer, offset, size - offset)
+        : DxvkBufferSlice();
     }
     
     DxvkBufferSlice GetBufferSlice(VkDeviceSize offset, VkDeviceSize length) const {
-      return DxvkBufferSlice(m_buffer, offset, length);
+      VkDeviceSize size = m_desc.ByteWidth;
+
+      return likely(offset < size)
+        ? DxvkBufferSlice(m_buffer, offset, std::min(length, size - offset))
+        : DxvkBufferSlice();
     }
 
     DxvkBufferSlice GetSOCounter() {
@@ -111,6 +118,21 @@ namespace dxvk {
       return &m_d3d10;
     }
 
+    bool HasSequenceNumber() const {
+      return m_mapMode != D3D11_COMMON_BUFFER_MAP_MODE_NONE
+          && !(m_desc.MiscFlags & D3D11_RESOURCE_MISC_DRAWINDIRECT_ARGS)
+          && !(m_desc.BindFlags);
+    }
+
+    void TrackSequenceNumber(uint64_t Seq) {
+      m_seq = Seq;
+    }
+
+    uint64_t GetSequenceNumber() {
+      return HasSequenceNumber() ? m_seq
+        : DxvkCsThread::SynchronizeAll;
+    }
+
     /**
      * \brief Normalizes buffer description
      * 
@@ -122,14 +144,16 @@ namespace dxvk {
 
   private:
     
-    const D3D11_BUFFER_DESC     m_desc;
+    D3D11_BUFFER_DESC             m_desc;
+    D3D11_COMMON_BUFFER_MAP_MODE  m_mapMode;
     
-    Rc<DxvkBuffer>              m_buffer;
-    Rc<DxvkBuffer>              m_soCounter;
-    DxvkBufferSliceHandle       m_mapped;
+    Rc<DxvkBuffer>                m_buffer;
+    Rc<DxvkBuffer>                m_soCounter;
+    DxvkBufferSliceHandle         m_mapped;
+    uint64_t                      m_seq = 0ull;
 
-    D3D11DXGIResource           m_resource;
-    D3D10Buffer                 m_d3d10;
+    D3D11DXGIResource             m_resource;
+    D3D10Buffer                   m_d3d10;
 
     BOOL CheckFormatFeatureSupport(
             VkFormat              Format,
@@ -139,6 +163,8 @@ namespace dxvk {
 
     Rc<DxvkBuffer> CreateSoCounterBuffer();
 
+    D3D11_COMMON_BUFFER_MAP_MODE DetermineMapMode();
+
   };
 
 
