@@ -18,9 +18,6 @@ namespace dxvk {
             REFIID  riid,
             void**  ppvObject) final;
     
-    void STDMETHODCALLTYPE GetDevice(
-            ID3D11Device **ppDevice) final;
-    
     UINT STDMETHODCALLTYPE GetContextFlags() final;
     
     void AddChunk(
@@ -32,22 +29,31 @@ namespace dxvk {
     void EmitToCommandList(
             ID3D11CommandList*  pCommandList);
     
-    void EmitToCsThread(
+    uint64_t EmitToCsThread(
             DxvkCsThread*       CsThread);
-    
+
+    void TrackResourceUsage(
+            ID3D11Resource*     pResource,
+            D3D11_RESOURCE_DIMENSION ResourceType,
+            UINT                Subresource);
+
   private:
-    
-    D3D11Device* const m_device;
+
     UINT         const m_contextFlags;
     
     std::vector<DxvkCsChunkRef>         m_chunks;
     std::vector<Com<D3D11Query, false>> m_queries;
+    std::vector<D3D11ResourceRef>       m_resources;
 
     std::atomic<bool> m_submitted = { false };
     std::atomic<bool> m_warned    = { false };
 
+    void TrackResourceSequenceNumber(
+      const D3D11ResourceRef&   Resource,
+            uint64_t            Seq);
+
     void MarkSubmitted();
     
   };
   
-}
\ No newline at end of file
+}
