@@ -7,7 +7,8 @@ namespace dxvk {
   D3D11SamplerState::D3D11SamplerState(
           D3D11Device*        device,
     const D3D11_SAMPLER_DESC& desc)
-  : m_device(device), m_desc(desc), m_d3d10(this) {
+  : D3D11StateObject<ID3D11SamplerState>(device),
+    m_desc(desc), m_d3d10(this) {
     DxvkSamplerCreateInfo info;
     
     // While D3D11_FILTER is technically an enum, its value bits
@@ -37,6 +38,7 @@ namespace dxvk {
       info.borderColor.float32[i] = desc.BorderColor[i];
     
     info.usePixelCoord  = VK_FALSE;  // Not supported in D3D11
+    info.nonSeamless    = VK_FALSE;
     
     // Make sure to use a valid anisotropy value
     if (desc.MaxAnisotropy <  1) info.maxAnisotropy =  1.0f;
@@ -45,7 +47,7 @@ namespace dxvk {
     // Enforce anisotropy specified in the device options
     int32_t samplerAnisotropyOption = device->GetOptions()->samplerAnisotropy;
 
-    if (samplerAnisotropyOption >= 0) {
+    if (samplerAnisotropyOption >= 0 && info.minFilter == VK_FILTER_LINEAR) {
       info.useAnisotropy = samplerAnisotropyOption > 0;
       info.maxAnisotropy = float(samplerAnisotropyOption);
     }
@@ -57,22 +59,6 @@ namespace dxvk {
   D3D11SamplerState::~D3D11SamplerState() {
     
   }
-  
-  
-  ULONG STDMETHODCALLTYPE D3D11SamplerState::AddRef() {
-    ULONG refCount = m_refCount++;
-    if (!refCount)
-      m_device->AddRef();
-    return refCount + 1;
-  }
-
-
-  ULONG STDMETHODCALLTYPE D3D11SamplerState::Release() {
-    ULONG refCount = --m_refCount;
-    if (!refCount)
-      m_device->Release();
-    return refCount;
-  }
 
 
   HRESULT STDMETHODCALLTYPE D3D11SamplerState::QueryInterface(REFIID riid, void** ppvObject) {
@@ -100,11 +86,6 @@ namespace dxvk {
   }
   
   
-  void STDMETHODCALLTYPE D3D11SamplerState::GetDevice(ID3D11Device** ppDevice) {
-    *ppDevice = ref(m_device);
-  }
-  
-  
   void STDMETHODCALLTYPE D3D11SamplerState::GetDesc(D3D11_SAMPLER_DESC* pDesc) {
     *pDesc = m_desc;
   }
