@@ -2,15 +2,19 @@
 #include "d3d11_gdi.h"
 #include "d3d11_texture.h"
 
+#include "../util/util_shared_res.h"
+
 namespace dxvk {
   
   D3D11CommonTexture::D3D11CommonTexture(
+          ID3D11Resource*             pInterface,
           D3D11Device*                pDevice,
     const D3D11_COMMON_TEXTURE_DESC*  pDesc,
           D3D11_RESOURCE_DIMENSION    Dimension,
           DXGI_USAGE                  DxgiUsage,
-          VkImage                     vkImage)
-  : m_device(pDevice), m_dimension(Dimension), m_desc(*pDesc), m_dxgiUsage(DxgiUsage) {
+          VkImage                     vkImage,
+          HANDLE                      hSharedHandle)
+  : m_interface(pInterface), m_device(pDevice), m_dimension(Dimension), m_desc(*pDesc), m_dxgiUsage(DxgiUsage) {
     DXGI_VK_FORMAT_MODE   formatMode   = GetFormatMode();
     DXGI_VK_FORMAT_INFO   formatInfo   = m_device->LookupFormat(m_desc.Format, formatMode);
     DXGI_VK_FORMAT_FAMILY formatFamily = m_device->LookupFamily(m_desc.Format, formatMode);
@@ -37,21 +41,30 @@ namespace dxvk {
     imageInfo.initialLayout   = VK_IMAGE_LAYOUT_UNDEFINED;
     imageInfo.shared          = vkImage != VK_NULL_HANDLE;
 
+    // Normalise hSharedhandle to INVALID_HANDLE_VALUE to allow passing in nullptr
+    if (hSharedHandle == nullptr)
+      hSharedHandle = INVALID_HANDLE_VALUE;
+
+    if (m_desc.MiscFlags & (D3D11_RESOURCE_MISC_SHARED|D3D11_RESOURCE_MISC_SHARED_NTHANDLE)) {
+      if (m_desc.MiscFlags & D3D11_RESOURCE_MISC_SHARED_KEYEDMUTEX)
+        Logger::warn("D3D11_RESOURCE_MISC_SHARED_KEYEDMUTEX: not supported.");
+
+      imageInfo.shared = true;
+      imageInfo.sharing.mode = hSharedHandle == INVALID_HANDLE_VALUE ? DxvkSharedHandleMode::Export : DxvkSharedHandleMode::Import;
+      imageInfo.sharing.type = m_desc.MiscFlags & D3D11_RESOURCE_MISC_SHARED_NTHANDLE
+        ? VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_BIT
+        : VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_KMT_BIT;
+      imageInfo.sharing.handle = hSharedHandle;
+    }
+
     if (!pDevice->GetOptions()->disableMsaa)
       DecodeSampleCount(m_desc.SampleDesc.Count, &imageInfo.sampleCount);
 
-    // Integer clear operations on UAVs are implemented using
-    // a view with a bit-compatible integer format, so we'll
-    // have to include that format in the format family
-    if (m_desc.BindFlags & D3D11_BIND_UNORDERED_ACCESS) {
-      DXGI_VK_FORMAT_INFO formatBase = m_device->LookupFormat(
-        m_desc.Format, DXGI_VK_FORMAT_MODE_RAW);
-
-      if (formatBase.Format != formatInfo.Format
-       && formatBase.Format != VK_FORMAT_UNDEFINED) {
-        formatFamily.Add(formatBase.Format);
-        formatFamily.Add(formatInfo.Format);
-      }
+    if ((m_desc.BindFlags & D3D11_BIND_UNORDERED_ACCESS) && IsR32UavCompatibleFormat(m_desc.Format)) {
+      formatFamily.Add(formatInfo.Format);
+      formatFamily.Add(VK_FORMAT_R32_SFLOAT);
+      formatFamily.Add(VK_FORMAT_R32_UINT);
+      formatFamily.Add(VK_FORMAT_R32_SINT);
     }
 
     // The image must be marked as mutable if it can be reinterpreted
@@ -59,19 +72,14 @@ namespace dxvk {
     // be reinterpreted in Vulkan, so we'll ignore those.
     auto formatProperties = imageFormatInfo(formatInfo.Format);
     
-    bool isTypeless = formatInfo.Aspect == 0;
     bool isMutable = formatFamily.FormatCount > 1;
     bool isMultiPlane = (formatProperties->aspectMask & VK_IMAGE_ASPECT_PLANE_0_BIT) != 0;
     bool isColorFormat = (formatProperties->aspectMask & VK_IMAGE_ASPECT_COLOR_BIT) != 0;
 
     if (isMutable && (isColorFormat || isMultiPlane)) {
       imageInfo.flags |= VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT;
-
-      // Typeless UAV images have relaxed reinterpretation rules
-      if (!isTypeless || !(m_desc.BindFlags & D3D11_BIND_UNORDERED_ACCESS)) {
-        imageInfo.viewFormatCount = formatFamily.FormatCount;
-        imageInfo.viewFormats     = formatFamily.Formats;
-      }
+      imageInfo.viewFormatCount = formatFamily.FormatCount;
+      imageInfo.viewFormats     = formatFamily.Formats;
     }
 
     // Adjust image flags based on the corresponding D3D flags
@@ -162,7 +170,7 @@ namespace dxvk {
           if (m_mapMode != D3D11_COMMON_TEXTURE_MAP_MODE_DIRECT)
             m_buffers.push_back(CreateMappedBuffer(j));
 
-          m_mapTypes.push_back(D3D11_MAP(~0u));
+          m_mapInfo.push_back({ D3D11_MAP(~0u), 0ull });
         }
       }
     }
@@ -174,7 +182,7 @@ namespace dxvk {
     // We must keep LINEAR images in GENERAL layout, but we
     // can choose a better layout for the image based on how
     // it is going to be used by the game.
-    if (imageInfo.tiling == VK_IMAGE_TILING_OPTIMAL && !isMultiPlane)
+    if (imageInfo.tiling == VK_IMAGE_TILING_OPTIMAL && !isMultiPlane && imageInfo.sharing.mode == DxvkSharedHandleMode::None)
       imageInfo.layout = OptimizeLayout(imageInfo.usage);
 
     // For some formats, we need to enable sampled and/or
@@ -202,16 +210,16 @@ namespace dxvk {
     // in case it is going to be mapped directly.
     VkMemoryPropertyFlags memoryProperties = VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
     
-    if (m_mapMode == D3D11_COMMON_TEXTURE_MAP_MODE_DIRECT) {
-      memoryProperties = VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT
-                       | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT
-                       | VK_MEMORY_PROPERTY_HOST_CACHED_BIT;
-    }
+    if (m_mapMode == D3D11_COMMON_TEXTURE_MAP_MODE_DIRECT)
+      memoryProperties = GetMemoryFlags();
     
     if (vkImage == VK_NULL_HANDLE)
       m_image = m_device->GetDXVKDevice()->createImage(imageInfo, memoryProperties);
     else
       m_image = m_device->GetDXVKDevice()->createImageFromVkImage(imageInfo, vkImage);
+
+    if (imageInfo.sharing.mode == DxvkSharedHandleMode::Export)
+      ExportImageInfo();
   }
   
   
@@ -412,7 +420,7 @@ namespace dxvk {
 
     // Use the maximum possible mip level count if the supplied
     // mip level count is either unspecified (0) or invalid
-    const uint32_t maxMipLevelCount = pDesc->SampleDesc.Count <= 1
+    const uint32_t maxMipLevelCount = (pDesc->SampleDesc.Count <= 1)
       ? util::computeMipLevelCount({ pDesc->Width, pDesc->Height, pDesc->Depth })
       : 1u;
     
@@ -530,6 +538,22 @@ namespace dxvk {
   }
 
   
+  VkMemoryPropertyFlags D3D11CommonTexture::GetMemoryFlags() const {
+    VkMemoryPropertyFlags memoryFlags = VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT
+                                      | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT;
+
+    bool useCached = (m_device->GetOptions()->cachedDynamicResources == ~0u)
+                  || (m_device->GetOptions()->cachedDynamicResources & m_desc.BindFlags);
+
+    if (m_desc.Usage == D3D11_USAGE_STAGING || useCached)
+      memoryFlags |= VK_MEMORY_PROPERTY_HOST_CACHED_BIT;
+    else if (m_desc.Usage == D3D11_USAGE_DEFAULT || m_desc.BindFlags)
+      memoryFlags |= VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT;
+
+    return memoryFlags;
+  }
+
+
   D3D11_COMMON_TEXTURE_MAP_MODE D3D11CommonTexture::DetermineMapMode(
     const DxvkImageCreateInfo*  pImageInfo) const {
     // Don't map an image unless the application requests it
@@ -542,16 +566,6 @@ namespace dxvk {
     if (!m_desc.BindFlags && m_desc.Usage != D3D11_USAGE_DEFAULT)
       return D3D11_COMMON_TEXTURE_MAP_MODE_STAGING;
 
-    // Write-only images should go through a buffer for multiple reasons:
-    // 1. Some games do not respect the row and depth pitch that is returned
-    //    by the Map() method, which leads to incorrect rendering (e.g. Nier)
-    // 2. Since the image will most likely be read for rendering by the GPU,
-    //    writing the image to device-local image may be more efficient than
-    //    reading its contents from host memory.
-    if (m_desc.Usage         == D3D11_USAGE_DYNAMIC
-     && m_desc.TextureLayout != D3D11_TEXTURE_LAYOUT_ROW_MAJOR)
-      return D3D11_COMMON_TEXTURE_MAP_MODE_BUFFER;
-    
     // Depth-stencil formats in D3D11 can be mapped and follow special
     // packing rules, so we need to copy that data into a buffer first
     if (GetPackedDepthStencilFormat(m_desc.Format))
@@ -561,16 +575,74 @@ namespace dxvk {
     if (imageFormatInfo(pImageInfo->format)->flags.test(DxvkFormatFlag::MultiPlane))
       return D3D11_COMMON_TEXTURE_MAP_MODE_BUFFER;
 
-    // Images that can be read by the host should be mapped directly in
-    // order to avoid expensive synchronization with the GPU. This does
-    // however require linear tiling, which may not be supported for all
-    // combinations of image parameters.
-    return this->CheckImageSupport(pImageInfo, VK_IMAGE_TILING_LINEAR)
-      ? D3D11_COMMON_TEXTURE_MAP_MODE_DIRECT
-      : D3D11_COMMON_TEXTURE_MAP_MODE_BUFFER;
+    // If we can't use linear tiling for this image, we have to use a buffer
+    if (!this->CheckImageSupport(pImageInfo, VK_IMAGE_TILING_LINEAR))
+      return D3D11_COMMON_TEXTURE_MAP_MODE_BUFFER;
+
+    // If supported and requested, create a linear image. Default images
+    // can be used for resolves and other operations regardless of bind
+    // flags, so we need to use a proper image for those.
+    if (m_desc.TextureLayout == D3D11_TEXTURE_LAYOUT_ROW_MAJOR
+     || m_desc.Usage         == D3D11_USAGE_DEFAULT)
+      return D3D11_COMMON_TEXTURE_MAP_MODE_DIRECT;
+
+    // The overhead of frequently uploading large dynamic images may outweigh
+    // the benefit of linear tiling, so use a linear image in those cases.
+    VkDeviceSize threshold = m_device->GetOptions()->maxDynamicImageBufferSize;
+    VkDeviceSize size = util::computeImageDataSize(pImageInfo->format, pImageInfo->extent);
+
+    if (size > threshold)
+      return D3D11_COMMON_TEXTURE_MAP_MODE_DIRECT;
+
+    // Dynamic images that can be sampled by a shader should generally go
+    // through a buffer to allow optimal tiling and to avoid running into
+    // bugs where games ignore the pitch when mapping the image.
+    return D3D11_COMMON_TEXTURE_MAP_MODE_BUFFER;
   }
   
   
+  void D3D11CommonTexture::ExportImageInfo() {
+    HANDLE hSharedHandle;
+
+    if (m_desc.MiscFlags & D3D11_RESOURCE_MISC_SHARED)
+      hSharedHandle = openKmtHandle( m_image->sharedHandle() );
+    else
+      hSharedHandle = m_image->sharedHandle();
+
+    DxvkSharedTextureMetadata metadata;
+
+    metadata.Width          = m_desc.Width;
+    metadata.Height         = m_desc.Height;
+    metadata.MipLevels      = m_desc.MipLevels;
+    metadata.ArraySize      = m_desc.ArraySize;
+    metadata.Format         = m_desc.Format;
+    metadata.SampleDesc     = m_desc.SampleDesc;
+    metadata.Usage          = m_desc.Usage;
+    metadata.BindFlags      = m_desc.BindFlags;
+    metadata.CPUAccessFlags = m_desc.CPUAccessFlags;
+    metadata.MiscFlags      = m_desc.MiscFlags;
+    metadata.TextureLayout  = m_desc.TextureLayout;
+
+    if (hSharedHandle == INVALID_HANDLE_VALUE || !setSharedMetadata(hSharedHandle, &metadata, sizeof(metadata))) {
+      Logger::warn("D3D11: Failed to write shared resource info for a texture");
+    }
+
+    if (hSharedHandle != INVALID_HANDLE_VALUE)
+      CloseHandle(hSharedHandle);
+  }
+  
+  
+  BOOL D3D11CommonTexture::IsR32UavCompatibleFormat(
+          DXGI_FORMAT           Format) {
+    return Format == DXGI_FORMAT_R8G8B8A8_TYPELESS
+        || Format == DXGI_FORMAT_B8G8R8A8_TYPELESS
+        || Format == DXGI_FORMAT_B8G8R8X8_TYPELESS
+        || Format == DXGI_FORMAT_R10G10B10A2_TYPELESS
+        || Format == DXGI_FORMAT_R16G16_TYPELESS
+        || Format == DXGI_FORMAT_R32_TYPELESS;
+  }
+
+
   D3D11CommonTexture::MappedBuffer D3D11CommonTexture::CreateMappedBuffer(UINT MipLevel) const {
     const DxvkFormatInfo* formatInfo = imageFormatInfo(
       m_device->LookupPackedFormat(m_desc.Format, GetFormatMode()).Format);
@@ -592,7 +664,9 @@ namespace dxvk {
     VkMemoryPropertyFlags memType = VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT
                                   | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT;
     
-    if (m_desc.Usage == D3D11_USAGE_STAGING)
+    bool useCached = m_device->GetOptions()->cachedDynamicResources == ~0u;
+
+    if (m_desc.Usage == D3D11_USAGE_STAGING || useCached)
       memType |= VK_MEMORY_PROPERTY_HOST_CACHED_BIT;
     
     MappedBuffer result;
@@ -910,7 +984,7 @@ namespace dxvk {
           D3D11Device*                pDevice,
     const D3D11_COMMON_TEXTURE_DESC*  pDesc)
   : D3D11DeviceChild<ID3D11Texture1D>(pDevice),
-    m_texture (pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE1D, 0, VK_NULL_HANDLE),
+    m_texture (this, pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE1D, 0, VK_NULL_HANDLE, nullptr),
     m_interop (this, &m_texture),
     m_surface (this, &m_texture),
     m_resource(this),
@@ -1006,14 +1080,15 @@ namespace dxvk {
   //      D 3 D 1 1 T E X T U R E 2 D
   D3D11Texture2D::D3D11Texture2D(
           D3D11Device*                pDevice,
-    const D3D11_COMMON_TEXTURE_DESC*  pDesc)
+    const D3D11_COMMON_TEXTURE_DESC*  pDesc,
+          HANDLE                      hSharedHandle)
   : D3D11DeviceChild<ID3D11Texture2D1>(pDevice),
-    m_texture (pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE2D, 0, VK_NULL_HANDLE),
-    m_interop (this, &m_texture),
-    m_surface (this, &m_texture),
-    m_resource(this),
-    m_d3d10   (this) {
-    
+    m_texture   (this, pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE2D, 0, VK_NULL_HANDLE, hSharedHandle),
+    m_interop   (this, &m_texture),
+    m_surface   (this, &m_texture),
+    m_resource  (this),
+    m_d3d10     (this),
+    m_swapChain (nullptr) {
   }
 
 
@@ -1023,11 +1098,28 @@ namespace dxvk {
           DXGI_USAGE                  DxgiUsage,
           VkImage                     vkImage)
   : D3D11DeviceChild<ID3D11Texture2D1>(pDevice),
-    m_texture (pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE2D, DxgiUsage, vkImage),
-    m_interop (this, &m_texture),
-    m_surface (this, &m_texture),
-    m_resource(this),
-    m_d3d10   (this) {
+    m_texture   (this, pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE2D, DxgiUsage, vkImage, nullptr),
+    m_interop   (this, &m_texture),
+    m_surface   (this, &m_texture),
+    m_resource  (this),
+    m_d3d10     (this),
+    m_swapChain (nullptr) {
+    
+  }
+
+
+  D3D11Texture2D::D3D11Texture2D(
+          D3D11Device*                pDevice,
+          IUnknown*                   pSwapChain,
+    const D3D11_COMMON_TEXTURE_DESC*  pDesc,
+          DXGI_USAGE                  DxgiUsage)
+  : D3D11DeviceChild<ID3D11Texture2D1>(pDevice),
+    m_texture   (this, pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE2D, DxgiUsage, VK_NULL_HANDLE, nullptr),
+    m_interop   (this, &m_texture),
+    m_surface   (this, &m_texture),
+    m_resource  (this),
+    m_d3d10     (this),
+    m_swapChain (pSwapChain) {
     
   }
   
@@ -1037,6 +1129,34 @@ namespace dxvk {
   }
   
   
+  ULONG STDMETHODCALLTYPE D3D11Texture2D::AddRef() {
+    uint32_t refCount = m_refCount++;
+
+    if (unlikely(!refCount)) {
+      if (m_swapChain)
+        m_swapChain->AddRef();
+
+      AddRefPrivate();
+    }
+
+    return refCount + 1;
+  }
+  
+
+  ULONG STDMETHODCALLTYPE D3D11Texture2D::Release() {
+    uint32_t refCount = --m_refCount;
+
+    if (unlikely(!refCount)) {
+      if (m_swapChain)
+        m_swapChain->Release();
+
+      ReleasePrivate();
+    }
+
+    return refCount;
+  }
+
+
   HRESULT STDMETHODCALLTYPE D3D11Texture2D::QueryInterface(REFIID riid, void** ppvObject) {
     if (ppvObject == nullptr)
       return E_POINTER;
@@ -1139,7 +1259,7 @@ namespace dxvk {
           D3D11Device*                pDevice,
     const D3D11_COMMON_TEXTURE_DESC*  pDesc)
   : D3D11DeviceChild<ID3D11Texture3D1>(pDevice),
-    m_texture (pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE3D, 0, VK_NULL_HANDLE),
+    m_texture (this, pDevice, pDesc, D3D11_RESOURCE_DIMENSION_TEXTURE3D, 0, VK_NULL_HANDLE, nullptr),
     m_interop (this, &m_texture),
     m_resource(this),
     m_d3d10   (this) {
