@@ -45,8 +45,7 @@ namespace dxvk {
 
   template <size_t N>
   static void copyToStringArray(char (&dst)[N], const char* src) {
-    std::strncpy(dst, src, N);
-    dst[N - 1] = '\0';
+    dxvk::str::strlcpy(dst, src, N);
   }
 
 
@@ -237,7 +236,8 @@ namespace dxvk {
           D3DDEVTYPE DeviceType,
           D3D9Format SourceFormat,
           D3D9Format TargetFormat) {
-    bool sourceSupported = IsSupportedBackBufferFormat(D3D9Format::Unknown, SourceFormat, TRUE);
+    bool sourceSupported = SourceFormat != D3D9Format::Unknown
+                        && IsSupportedBackBufferFormat(SourceFormat);
     bool targetSupported = TargetFormat == D3D9Format::X1R5G5B5
                         || TargetFormat == D3D9Format::A1R5G5B5
                         || TargetFormat == D3D9Format::R5G6B5
