@@ -1,6 +1,6 @@
 #pragma once
 
-#include "d3d9_include.h"
+#include <d3d9.h>
 #include "d3d9_options.h"
 
 #include "../dxvk/dxvk_adapter.h"
@@ -165,7 +165,7 @@ namespace dxvk {
       VK_COMPONENT_SWIZZLE_IDENTITY, VK_COMPONENT_SWIZZLE_IDENTITY };
     D3D9_CONVERSION_FORMAT_INFO ConversionFormatInfo = { };
 
-    bool IsValid() { return FormatColor != VK_FORMAT_UNDEFINED; }
+    bool IsValid() const { return FormatColor != VK_FORMAT_UNDEFINED; }
   };
 
   D3D9_VK_FORMAT_MAPPING ConvertFormatUnfixed(D3D9Format Format);
@@ -201,7 +201,7 @@ namespace dxvk {
      *
      * \param [in] Format The D3D9 format to look up
      */
-    DxvkFormatInfo GetUnsupportedFormatInfo(
+    const DxvkFormatInfo* GetUnsupportedFormatInfo(
       D3D9Format            Format) const;
 
   private:
@@ -220,4 +220,4 @@ namespace dxvk {
     bool m_d32supportFinal;
   };
 
-}
\ No newline at end of file
+}
