@@ -10,6 +10,7 @@ namespace dxvk {
   struct D3D9WindowData {
     bool unicode;
     bool filter;
+    bool activateProcessed;
     WNDPROC proc;
     D3D9SwapChainEx* swapchain;
   };
@@ -18,6 +19,13 @@ namespace dxvk {
   static dxvk::recursive_mutex g_windowProcMapMutex;
   static std::unordered_map<HWND, D3D9WindowData> g_windowProcMap;
 
+  static void SetActivateProcessed(HWND window, bool processed)
+  {
+      std::lock_guard lock(g_windowProcMapMutex);
+      auto it = g_windowProcMap.find(window);
+      if (it != g_windowProcMap.end())
+        it->second.activateProcessed = processed;
+  }
 
   template <typename T, typename J, typename ... Args>
   auto CallCharsetFunction(T unicode, J ascii, bool isUnicode, Args... args) {
@@ -88,6 +96,7 @@ namespace dxvk {
     D3D9WindowData windowData;
     windowData.unicode = IsWindowUnicode(window);
     windowData.filter  = false;
+    windowData.activateProcessed = false;
     windowData.proc = reinterpret_cast<WNDPROC>(
       CallCharsetFunction(
       SetWindowLongPtrW, SetWindowLongPtrA, windowData.unicode,
@@ -128,7 +137,8 @@ namespace dxvk {
       windowData.swapchain->GetDevice()->GetCreationParameters(&create_parms);
 
       if (!(create_parms.BehaviorFlags & D3DCREATE_NOWINDOWCHANGES)) {
-        if (wparam) {
+        D3D9WindowMessageFilter filter(window);
+        if (wparam && !windowData.activateProcessed) {
           // Heroes of Might and Magic V needs this to resume drawing after a focus loss
           D3DPRESENT_PARAMETERS params;
           RECT rect;
@@ -137,10 +147,12 @@ namespace dxvk {
           windowData.swapchain->GetPresentParameters(&params);
           SetWindowPos(window, nullptr, rect.left, rect.top, params.BackBufferWidth, params.BackBufferHeight,
                        SWP_NOACTIVATE | SWP_NOZORDER);
+          SetActivateProcessed(window, true);
         }
-        else {
+        else if (!wparam) {
           if (IsWindowVisible(window))
             ShowWindow(window, SW_MINIMIZE);
+          SetActivateProcessed(window, false);
         }
       }
     }
@@ -187,7 +199,7 @@ namespace dxvk {
     m_window = m_presentParams.hDeviceWindow;
 
     UpdatePresentRegion(nullptr, nullptr);
-    if (!pDevice->GetOptions()->deferSurfaceCreation)
+    if (m_window && !pDevice->GetOptions()->deferSurfaceCreation)
       CreatePresenter();
 
     CreateBackBuffers(m_presentParams.BackBufferCount);
@@ -321,7 +333,7 @@ namespace dxvk {
     D3D9CommonTexture* dstTexInfo = dst->GetCommonTexture();
     D3D9CommonTexture* srcTexInfo = m_backBuffers.back()->GetCommonTexture();
 
-    if (unlikely(dstTexInfo->Desc()->Pool != D3DPOOL_SYSTEMMEM))
+    if (unlikely(dstTexInfo->Desc()->Pool != D3DPOOL_SYSTEMMEM && dstTexInfo->Desc()->Pool != D3DPOOL_SCRATCH))
       return D3DERR_INVALIDCALL;
 
     Rc<DxvkBuffer> dstBuffer = dstTexInfo->GetBuffer(dst->GetSubresource());
@@ -468,8 +480,9 @@ namespace dxvk {
         cImage, cSubresources, VkOffset3D { 0, 0, 0 },
         cLevelExtent);
     });
-    
-    dstTexInfo->SetWrittenByGPU(dst->GetSubresource(), true);
+
+    dstTexInfo->SetNeedsReadback(dst->GetSubresource(), true);
+    m_parent->TrackTextureMappingBufferSequenceNumber(dstTexInfo, dst->GetSubresource());
 
     return D3D_OK;
   }
@@ -618,21 +631,6 @@ namespace dxvk {
     if (pPresentParams->Windowed) {
       if (changeFullscreen)
         this->LeaveFullscreenMode();
-
-      // Adjust window position and size
-      RECT newRect = { 0, 0, 0, 0 };
-      RECT oldRect = { 0, 0, 0, 0 };
-      
-      ::GetWindowRect(m_window, &oldRect);
-      ::MapWindowPoints(HWND_DESKTOP, ::GetParent(m_window), reinterpret_cast<POINT*>(&oldRect), 1);
-      ::SetRect(&newRect, 0, 0, pPresentParams->BackBufferWidth, pPresentParams->BackBufferHeight);
-      ::AdjustWindowRectEx(&newRect,
-        ::GetWindowLongW(m_window, GWL_STYLE), FALSE,
-        ::GetWindowLongW(m_window, GWL_EXSTYLE));
-      ::SetRect(&newRect, 0, 0, newRect.right - newRect.left, newRect.bottom - newRect.top);
-      ::OffsetRect(&newRect, oldRect.left, oldRect.top);    
-      ::MoveWindow(m_window, newRect.left, newRect.top,
-        newRect.right - newRect.left, newRect.bottom - newRect.top, TRUE);
     }
     else {
       if (changeFullscreen) {
@@ -996,7 +994,7 @@ namespace dxvk {
       VkImage imageHandle = m_presenter->getImage(i).image;
       
       Rc<DxvkImage> image = new DxvkImage(
-        m_device->vkd(), imageInfo, imageHandle);
+        m_device.ptr(), imageInfo, imageHandle);
 
       m_imageViews[i] = new DxvkImageView(
         m_device->vkd(), image, viewInfo);
@@ -1037,7 +1035,7 @@ namespace dxvk {
     desc.IsAttachmentOnly   = FALSE;
 
     for (uint32_t i = 0; i < m_backBuffers.size(); i++)
-      m_backBuffers[i] = new D3D9Surface(m_parent, &desc, this);
+      m_backBuffers[i] = new D3D9Surface(m_parent, &desc, this, nullptr);
 
     auto swapImage = m_backBuffers[0]->GetCommonTexture()->GetImage();
 
@@ -1050,19 +1048,13 @@ namespace dxvk {
     subresources.baseArrayLayer = 0;
     subresources.layerCount     = 1;
 
-    VkClearColorValue clearColor;
-    clearColor.float32[0] = 0.0f;
-    clearColor.float32[1] = 0.0f;
-    clearColor.float32[2] = 0.0f;
-    clearColor.float32[3] = 0.0f;
-
     m_context->beginRecording(
       m_device->createCommandList());
     
     for (uint32_t i = 0; i < m_backBuffers.size(); i++) {
-      m_context->clearColorImage(
+      m_context->initImage(
         m_backBuffers[i]->GetCommonTexture()->GetImage(),
-        clearColor, subresources);
+        subresources, VK_IMAGE_LAYOUT_UNDEFINED);
     }
 
     m_device->submitCommandList(
@@ -1122,8 +1114,9 @@ namespace dxvk {
 
     switch (Format) {
       default:
-        Logger::warn(str::format("D3D9SwapChainEx: Unexpected format: ", Format));
-        
+        Logger::warn(str::format("D3D9SwapChainEx: Unexpected format: ", Format));      
+     [[fallthrough]];
+
       case D3D9Format::A8R8G8B8:
       case D3D9Format::X8R8G8B8:
       case D3D9Format::A8B8G8R8:
@@ -1143,12 +1136,12 @@ namespace dxvk {
         pDstFormats[n++] = { VK_FORMAT_B5G5R5A1_UNORM_PACK16, VK_COLOR_SPACE_SRGB_NONLINEAR_KHR };
         pDstFormats[n++] = { VK_FORMAT_R5G5B5A1_UNORM_PACK16, VK_COLOR_SPACE_SRGB_NONLINEAR_KHR };
         pDstFormats[n++] = { VK_FORMAT_A1R5G5B5_UNORM_PACK16, VK_COLOR_SPACE_SRGB_NONLINEAR_KHR };
-      }
+      } break;
 
       case D3D9Format::R5G6B5: {
         pDstFormats[n++] = { VK_FORMAT_B5G6R5_UNORM_PACK16, VK_COLOR_SPACE_SRGB_NONLINEAR_KHR };
         pDstFormats[n++] = { VK_FORMAT_R5G6B5_UNORM_PACK16, VK_COLOR_SPACE_SRGB_NONLINEAR_KHR };
-      }
+      } break;
     }
 
     return n;
