@@ -31,12 +31,20 @@ namespace dxvk {
     // Fetch these as a string representing a hexadecimal number and parse it.
     this->customVendorId = parsePciId(config.getOption<std::string>("dxgi.customVendorId"));
     this->customDeviceId = parsePciId(config.getOption<std::string>("dxgi.customDeviceId"));
+    this->customDeviceDesc = config.getOption<std::string>("dxgi.customDeviceDesc", "");
+
+    // Emulate a UMA device
+    this->emulateUMA = config.getOption<bool>("dxgi.emulateUMA", false);
     
     // Interpret the memory limits as Megabytes
     this->maxDeviceMemory = VkDeviceSize(config.getOption<int32_t>("dxgi.maxDeviceMemory", 0)) << 20;
     this->maxSharedMemory = VkDeviceSize(config.getOption<int32_t>("dxgi.maxSharedMemory", 0)) << 20;
 
-    this->nvapiHack   = config.getOption<bool>("dxgi.nvapiHack", true);
+    // Force nvapiHack to be disabled if NvAPI is enabled in environment
+    if (env::getEnvVar("DXVK_ENABLE_NVAPI") == "1")
+      this->nvapiHack = false;
+    else
+      this->nvapiHack = config.getOption<bool>("dxgi.nvapiHack", true);
   }
   
-}
\ No newline at end of file
+}
