@@ -1,3 +1,4 @@
+#include "dxvk_barrier.h"
 #include "dxvk_buffer.h"
 #include "dxvk_device.h"
 
@@ -28,8 +29,10 @@ namespace dxvk {
       ? MaxBufferSize / m_physSliceStride
       : 1;
 
-    // Allocate the initial set of buffer slices
-    m_buffer = allocBuffer(m_physSliceCount);
+    // Allocate the initial set of buffer slices. Only clear
+    // buffer memory if there is more than one slice, since
+    // we expect the client api to initialize the first slice.
+    m_buffer = allocBuffer(m_physSliceCount, m_physSliceCount > 1);
 
     DxvkBufferSliceHandle slice;
     slice.handle = m_buffer.buffer;
@@ -51,7 +54,7 @@ namespace dxvk {
   }
   
   
-  DxvkBufferHandle DxvkBuffer::allocBuffer(VkDeviceSize sliceCount) const {
+  DxvkBufferHandle DxvkBuffer::allocBuffer(VkDeviceSize sliceCount, bool clear) const {
     auto vkd = m_device->vkd();
 
     VkBufferCreateInfo info;
@@ -108,6 +111,13 @@ namespace dxvk {
     if (isGpuWritable)
       hints.set(DxvkMemoryFlag::GpuWritable);
 
+    // Staging buffers that can't even be used as a transfer destinations
+    // are likely short-lived, so we should put them on a separate memory
+    // pool in order to avoid fragmentation
+    if ((DxvkBarrierSet::getAccessTypes(m_info.access) == DxvkAccess::Read)
+     && (m_info.usage & VK_BUFFER_USAGE_TRANSFER_SRC_BIT))
+      hints.set(DxvkMemoryFlag::Transient);
+
     // Ask driver whether we should be using a dedicated allocation
     handle.memory = m_memAlloc->alloc(&memReq.memoryRequirements,
       dedicatedRequirements, dedMemoryAllocInfo, m_memFlags, hints);
@@ -116,6 +126,9 @@ namespace dxvk {
         handle.memory.memory(), handle.memory.offset()) != VK_SUCCESS)
       throw DxvkError("DxvkBuffer: Failed to bind device memory");
     
+    if (clear && (m_memFlags & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT))
+      std::memset(handle.memory.mapPtr(0), 0, info.size);
+
     return handle;
   }
 
