@@ -232,7 +232,39 @@ namespace dxvk {
      * \brief Allocates new buffer slice
      * \returns The new buffer slice
      */
-    DxvkBufferSliceHandle allocSlice();
+    DxvkBufferSliceHandle allocSlice() {
+      std::unique_lock<sync::Spinlock> freeLock(m_freeMutex);
+      
+      // If no slices are available, swap the two free lists.
+      if (unlikely(m_freeSlices.empty())) {
+        std::unique_lock<sync::Spinlock> swapLock(m_swapMutex);
+        std::swap(m_freeSlices, m_nextSlices);
+      }
+
+      // If there are still no slices available, create a new
+      // backing buffer and add all slices to the free list.
+      if (unlikely(m_freeSlices.empty())) {
+        if (likely(!m_lazyAlloc)) {
+          DxvkBufferHandle handle = allocBuffer(m_physSliceCount, true);
+
+          for (uint32_t i = 0; i < m_physSliceCount; i++)
+            pushSlice(handle, i);
+
+          m_buffers.push_back(std::move(handle));
+          m_physSliceCount = std::min(m_physSliceCount * 2, m_physSliceMaxCount);
+        } else {
+          for (uint32_t i = 1; i < m_physSliceCount; i++)
+            pushSlice(m_buffer, i);
+
+          m_lazyAlloc = false;
+        }
+      }
+      
+      // Take the first slice from the queue
+      DxvkBufferSliceHandle result = m_freeSlices.back();
+      m_freeSlices.pop_back();
+      return result;
+    }
     
     /**
      * \brief Frees a buffer slice
@@ -242,8 +274,11 @@ namespace dxvk {
      * the slice is no longer needed by the GPU.
      * \param [in] slice The buffer slice to free
      */
-    void freeSlice(
-      const DxvkBufferSliceHandle& slice);
+    void freeSlice(const DxvkBufferSliceHandle& slice) {
+      // Add slice to a separate free list to reduce lock contention.
+      std::unique_lock<sync::Spinlock> swapLock(m_swapMutex);
+      m_nextSlices.push_back(slice);
+    }
     
   private:
 
@@ -254,22 +289,38 @@ namespace dxvk {
     
     DxvkBufferHandle        m_buffer;
     DxvkBufferSliceHandle   m_physSlice;
-
     uint32_t                m_vertexStride = 0;
-    
-    sync::Spinlock m_freeMutex;
-    sync::Spinlock m_swapMutex;
-    
-    std::vector<DxvkBufferHandle>        m_buffers;
-    std::vector<DxvkBufferSliceHandle>   m_freeSlices;
-    std::vector<DxvkBufferSliceHandle>   m_nextSlices;
-    
-    VkDeviceSize m_physSliceLength  = 0;
-    VkDeviceSize m_physSliceStride  = 0;
-    VkDeviceSize m_physSliceCount   = 2;
+
+    alignas(CACHE_LINE_SIZE)
+    sync::Spinlock          m_freeMutex;
+
+    uint32_t                m_lazyAlloc = false;
+    VkDeviceSize            m_physSliceLength   = 0;
+    VkDeviceSize            m_physSliceStride   = 0;
+    VkDeviceSize            m_physSliceCount    = 1;
+    VkDeviceSize            m_physSliceMaxCount = 1;
+
+    std::vector<DxvkBufferHandle>       m_buffers;
+    std::vector<DxvkBufferSliceHandle>  m_freeSlices;
+
+    alignas(CACHE_LINE_SIZE)
+    sync::Spinlock                      m_swapMutex;
+    std::vector<DxvkBufferSliceHandle>  m_nextSlices;
+
+    void pushSlice(const DxvkBufferHandle& handle, uint32_t index) {
+      DxvkBufferSliceHandle slice;
+      slice.handle = handle.buffer;
+      slice.length = m_physSliceLength;
+      slice.offset = m_physSliceStride * index;
+      slice.mapPtr = handle.memory.mapPtr(slice.offset);
+      m_freeSlices.push_back(slice);
+    }
 
     DxvkBufferHandle allocBuffer(
-            VkDeviceSize          sliceCount) const;
+            VkDeviceSize          sliceCount,
+            bool                  clear) const;
+
+    VkDeviceSize computeSliceAlignment() const;
     
   };
   
@@ -298,6 +349,23 @@ namespace dxvk {
     explicit DxvkBufferSlice(const Rc<DxvkBuffer>& buffer)
     : DxvkBufferSlice(buffer, 0, buffer->info().size) { }
 
+    DxvkBufferSlice(const DxvkBufferSlice& ) = default;
+    DxvkBufferSlice(      DxvkBufferSlice&&) = default;
+
+    DxvkBufferSlice& operator = (const DxvkBufferSlice& other) {
+      if (m_buffer != other.m_buffer)
+        m_buffer = other.m_buffer;
+      m_offset = other.m_offset;
+      m_length = other.m_length;
+      return *this;
+    }
+
+    DxvkBufferSlice& operator = (DxvkBufferSlice&&) = default;
+
+    /**
+     * \brief Buffer slice offset and length
+     * \returns Buffer slice offset and length
+     */
     size_t offset() const { return m_offset; }
     size_t length() const { return m_length; }
 
@@ -330,7 +398,7 @@ namespace dxvk {
      * \returns The sub slice object
      */
     DxvkBufferSlice subSlice(VkDeviceSize offset, VkDeviceSize length) const {
-      return DxvkBufferSlice(m_buffer, offset, length);
+      return DxvkBufferSlice(m_buffer, m_offset + offset, length);
     }
     
     /**
@@ -399,7 +467,7 @@ namespace dxvk {
         ? m_buffer->mapPtr(m_offset + offset)
         : nullptr;
     }
-    
+
     /**
      * \brief Checks whether two slices are equal
      * 
@@ -413,6 +481,32 @@ namespace dxvk {
           && this->m_offset == other.m_offset
           && this->m_length == other.m_length;
     }
+
+    /**
+     * \brief Checks whether two slices are from the same buffer
+     *
+     * This returns \c true if the two slices are taken
+     * from the same buffer, but may have different ranges.
+     * \param [in] other The slice to compare to
+     * \returns \c true if the buffer objects are the same
+     */
+    bool matchesBuffer(const DxvkBufferSlice& other) const {
+      return this->m_buffer == other.m_buffer;
+    }
+
+    /**
+     * \brief Checks whether two slices have the same range
+     * 
+     * This returns \c true if the two slices have the same
+     * offset and size, even if the buffers are different.
+     * May be useful if the buffers are know to be the same.
+     * \param [in] other The slice to compare to
+     * \returns \c true if the buffer objects are the same
+     */
+    bool matchesRange(const DxvkBufferSlice& other) const {
+      return this->m_offset == other.m_offset
+          && this->m_length == other.m_length;
+    }
     
   private:
     
@@ -485,6 +579,14 @@ namespace dxvk {
     const DxvkBufferCreateInfo& bufferInfo() const {
       return m_buffer->info();
     }
+    
+    /**
+     * \brief View format info
+     * \returns View format info
+     */
+    const DxvkFormatInfo* formatInfo() const {
+      return imageFormatInfo(m_info.format);
+    }
 
     /**
      * \brief Retrieves buffer slice handle
@@ -558,10 +660,21 @@ namespace dxvk {
     DxvkBufferTracker();
     ~DxvkBufferTracker();
     
-    void freeBufferSlice(
-      const Rc<DxvkBuffer>&         buffer,
-      const DxvkBufferSliceHandle&  slice);
+    /**
+     * \brief Add buffer slice for tracking
+     *
+     * The slice will be returned to the
+     * buffer on the next call to \c reset.
+     * \param [in] buffer The parent buffer
+     * \param [in] slice The buffer slice
+     */
+    void freeBufferSlice(const Rc<DxvkBuffer>& buffer, const DxvkBufferSliceHandle& slice) {
+      m_entries.push_back({ buffer, slice });
+    }
     
+    /**
+     * \brief Returns tracked buffer slices
+     */
     void reset();
     
   private:
