@@ -112,6 +112,12 @@ namespace dxvk {
   }
   
   
+  Rc<DxvkFence> DxvkDevice::createFence(
+    const DxvkFenceCreateInfo& fenceInfo) {
+    return new DxvkFence(this, fenceInfo);
+  }
+  
+  
   Rc<DxvkFramebuffer> DxvkDevice::createFramebuffer(
     const DxvkFramebufferInfo&  info) {
     return new DxvkFramebuffer(m_vkd, info);
@@ -135,14 +141,14 @@ namespace dxvk {
   Rc<DxvkImage> DxvkDevice::createImage(
     const DxvkImageCreateInfo&  createInfo,
           VkMemoryPropertyFlags memoryType) {
-    return new DxvkImage(m_vkd, createInfo, m_objects.memoryManager(), memoryType);
+    return new DxvkImage(this, createInfo, m_objects.memoryManager(), memoryType);
   }
   
   
   Rc<DxvkImage> DxvkDevice::createImageFromVkImage(
     const DxvkImageCreateInfo&  createInfo,
           VkImage               image) {
-    return new DxvkImage(m_vkd, createInfo, image);
+    return new DxvkImage(this, createInfo, image);
   }
   
   Rc<DxvkImageView> DxvkDevice::createImageView(
@@ -158,19 +164,6 @@ namespace dxvk {
   }
   
   
-  Rc<DxvkShader> DxvkDevice::createShader(
-          VkShaderStageFlagBits     stage,
-          uint32_t                  slotCount,
-    const DxvkResourceSlot*         slotInfos,
-    const DxvkInterfaceSlots&       iface,
-    const SpirvCodeBuffer&          code) {
-    return new DxvkShader(stage,
-      slotCount, slotInfos, iface, code,
-      DxvkShaderOptions(),
-      DxvkShaderConstData());
-  }
-  
-  
   DxvkStatCounters DxvkDevice::getStatCounters() {
     DxvkPipelineCount pipe = m_objects.pipelineManager().getPipelineCount();
     
@@ -246,6 +239,24 @@ namespace dxvk {
 
     return result;
   }
+
+
+  void DxvkDevice::waitForResource(const Rc<DxvkResource>& resource, DxvkAccess access) {
+    if (resource->isInUse(access)) {
+      auto t0 = dxvk::high_resolution_clock::now();
+
+      m_submissionQueue.synchronizeUntil([resource, access] {
+        return !resource->isInUse(access);
+      });
+
+      auto t1 = dxvk::high_resolution_clock::now();
+      auto us = std::chrono::duration_cast<std::chrono::microseconds>(t1 - t0);
+
+      std::lock_guard<sync::Spinlock> lock(m_statLock);
+      m_statCounters.addCtr(DxvkStatCounter::GpuSyncCount, 1);
+      m_statCounters.addCtr(DxvkStatCounter::GpuSyncTicks, us.count());
+    }
+  }
   
   
   void DxvkDevice::waitForIdle() {
