@@ -26,8 +26,8 @@ namespace dxvk {
     m_layout = new DxvkPipelineLayout(m_vkd,
       m_slotMapping, VK_PIPELINE_BIND_POINT_GRAPHICS);
     
-    m_vsIn  = m_shaders.vs != nullptr ? m_shaders.vs->interfaceSlots().inputSlots  : 0;
-    m_fsOut = m_shaders.fs != nullptr ? m_shaders.fs->interfaceSlots().outputSlots : 0;
+    m_vsIn  = m_shaders.vs != nullptr ? m_shaders.vs->info().inputMask  : 0;
+    m_fsOut = m_shaders.fs != nullptr ? m_shaders.fs->info().outputMask : 0;
 
     if (m_shaders.gs != nullptr && m_shaders.gs->flags().test(DxvkShaderFlag::HasTransformFeedback))
       m_flags.set(DxvkGraphicsPipelineFlag::HasTransformFeedback);
@@ -63,22 +63,25 @@ namespace dxvk {
   VkPipeline DxvkGraphicsPipeline::getPipelineHandle(
     const DxvkGraphicsPipelineStateInfo& state,
     const DxvkRenderPass*                renderPass) {
-    DxvkGraphicsPipelineInstance* instance = nullptr;
+    DxvkGraphicsPipelineInstance* instance = this->findInstance(state, renderPass);
 
-    { std::lock_guard<sync::Spinlock> lock(m_mutex);
-    
+    if (unlikely(!instance)) {
+      // Exit early if the state vector is invalid
+      if (!this->validatePipelineState(state, true))
+        return VK_NULL_HANDLE;
+
+      // Prevent other threads from adding new instances and check again
+      std::lock_guard<dxvk::mutex> lock(m_mutex);
       instance = this->findInstance(state, renderPass);
-      
-      if (instance)
-        return instance->pipeline();
-      
-      instance = this->createInstance(state, renderPass);
-    }
-    
-    if (!instance)
-      return VK_NULL_HANDLE;
 
-    this->writePipelineStateToCache(state, renderPass->format());
+      if (!instance) {
+        // Keep pipeline object locked, at worst we're going to stall
+        // a state cache worker and the current thread needs priority.
+        instance = this->createInstance(state, renderPass);
+        this->writePipelineStateToCache(state, renderPass->format());
+      }
+    }
+
     return instance->pipeline();
   }
 
@@ -86,7 +89,13 @@ namespace dxvk {
   void DxvkGraphicsPipeline::compilePipeline(
     const DxvkGraphicsPipelineStateInfo& state,
     const DxvkRenderPass*                renderPass) {
-    std::lock_guard<sync::Spinlock> lock(m_mutex);
+    // Exit early if the state vector is invalid
+    if (!this->validatePipelineState(state, false))
+      return;
+
+    // Keep the object locked while compiling a pipeline since compiling
+    // similar pipelines concurrently is fragile on some drivers
+    std::lock_guard<dxvk::mutex> lock(m_mutex);
 
     if (!this->findInstance(state, renderPass))
       this->createInstance(state, renderPass);
@@ -96,15 +105,10 @@ namespace dxvk {
   DxvkGraphicsPipelineInstance* DxvkGraphicsPipeline::createInstance(
     const DxvkGraphicsPipelineStateInfo& state,
     const DxvkRenderPass*                renderPass) {
-    // If the pipeline state vector is invalid, don't try
-    // to create a new pipeline, it won't work anyway.
-    if (!this->validatePipelineState(state))
-      return nullptr;
-
-    VkPipeline newPipelineHandle = this->createPipeline(state, renderPass);
+    VkPipeline pipeline = this->createPipeline(state, renderPass);
 
     m_pipeMgr->m_numGraphicsPipelines += 1;
-    return &m_pipelines.emplace_back(state, renderPass, newPipelineHandle);
+    return &(*m_pipelines.emplace(state, renderPass, pipeline));
   }
   
   
@@ -199,15 +203,24 @@ namespace dxvk {
       | VK_COLOR_COMPONENT_B_BIT | VK_COLOR_COMPONENT_A_BIT;
 
     for (uint32_t i = 0; i < MaxNumRenderTargets; i++) {
+      auto formatInfo = imageFormatInfo(passFormat.color[i].format);
       omBlendAttachments[i] = state.omBlend[i].state();
 
-      if (omBlendAttachments[i].colorWriteMask != fullMask) {
-        omBlendAttachments[i].colorWriteMask = util::remapComponentMask(
-          state.omBlend[i].colorWriteMask(), state.omSwizzle[i].mapping());
-      }
-      
-      if ((m_fsOut & (1 << i)) == 0)
+      if (!(m_fsOut & (1 << i)) || !formatInfo) {
         omBlendAttachments[i].colorWriteMask = 0;
+      } else {
+        if (omBlendAttachments[i].colorWriteMask != fullMask) {
+          omBlendAttachments[i].colorWriteMask = util::remapComponentMask(
+            state.omBlend[i].colorWriteMask(), state.omSwizzle[i].mapping());
+        }
+
+        omBlendAttachments[i].colorWriteMask &= formatInfo->componentMask;
+
+        if (omBlendAttachments[i].colorWriteMask == formatInfo->componentMask) {
+          omBlendAttachments[i].colorWriteMask = VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT
+                                               | VK_COLOR_COMPONENT_B_BIT | VK_COLOR_COMPONENT_A_BIT;
+        }
+      }
     }
 
     // Generate per-instance attribute divisors
@@ -225,7 +238,7 @@ namespace dxvk {
     }
 
     int32_t rasterizedStream = m_shaders.gs != nullptr
-      ? m_shaders.gs->shaderOptions().rasterizedStream
+      ? m_shaders.gs->info().xfbRasterizedStream
       : 0;
     
     // Compact vertex bindings so that we can more easily update vertex buffers
@@ -288,6 +301,13 @@ namespace dxvk {
     vpInfo.scissorCount           = state.rs.viewportCount();
     vpInfo.pScissors              = nullptr;
     
+    VkPipelineRasterizationConservativeStateCreateInfoEXT conservativeInfo;
+    conservativeInfo.sType        = VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_CONSERVATIVE_STATE_CREATE_INFO_EXT;
+    conservativeInfo.pNext        = nullptr;
+    conservativeInfo.flags        = 0;
+    conservativeInfo.conservativeRasterizationMode = state.rs.conservativeMode();
+    conservativeInfo.extraPrimitiveOverestimationSize = 0.0f;
+
     VkPipelineRasterizationStateStreamCreateInfoEXT xfbStreamInfo;
     xfbStreamInfo.sType           = VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_STREAM_CREATE_INFO_EXT;
     xfbStreamInfo.pNext           = nullptr;
@@ -302,7 +322,7 @@ namespace dxvk {
 
     VkPipelineRasterizationStateCreateInfo rsInfo;
     rsInfo.sType                  = VK_STRUCTURE_TYPE_PIPELINE_RASTERIZATION_STATE_CREATE_INFO;
-    rsInfo.pNext                  = &rsDepthClipInfo;
+    rsInfo.pNext                  = nullptr;
     rsInfo.flags                  = 0;
     rsInfo.depthClampEnable       = VK_TRUE;
     rsInfo.rasterizerDiscardEnable = rasterizedStream < 0;
@@ -316,12 +336,15 @@ namespace dxvk {
     rsInfo.lineWidth              = 1.0f;
     
     if (rasterizedStream > 0)
-      rsDepthClipInfo.pNext = &xfbStreamInfo;
-    
-    if (!m_pipeMgr->m_device->features().extDepthClipEnable.depthClipEnable) {
-      rsInfo.pNext                = rsDepthClipInfo.pNext;
-      rsInfo.depthClampEnable     = !state.rs.depthClipEnable();
-    }
+      xfbStreamInfo.pNext = std::exchange(rsInfo.pNext, &xfbStreamInfo);
+
+    if (conservativeInfo.conservativeRasterizationMode != VK_CONSERVATIVE_RASTERIZATION_MODE_DISABLED_EXT)
+      conservativeInfo.pNext = std::exchange(rsInfo.pNext, &conservativeInfo);
+
+    if (m_pipeMgr->m_device->features().extDepthClipEnable.depthClipEnable)
+      rsDepthClipInfo.pNext = std::exchange(rsInfo.pNext, &rsDepthClipInfo);
+    else
+      rsInfo.depthClampEnable = !state.rs.depthClipEnable();
 
     uint32_t sampleMask = state.ms.sampleMask();
 
@@ -428,10 +451,11 @@ namespace dxvk {
     if (shader == nullptr)
       return DxvkShaderModule();
 
+    const DxvkShaderCreateInfo& shaderInfo = shader->info();
     DxvkShaderModuleCreateInfo info;
 
     // Fix up fragment shader outputs for dual-source blending
-    if (shader->stage() == VK_SHADER_STAGE_FRAGMENT_BIT) {
+    if (shaderInfo.stage == VK_SHADER_STAGE_FRAGMENT_BIT) {
       info.fsDualSrcBlend = state.omBlend[0].blendEnable() && (
         util::isDualSourceBlendFactor(state.omBlend[0].srcColorBlendFactor()) ||
         util::isDualSourceBlendFactor(state.omBlend[0].dstColorBlendFactor()) ||
@@ -440,15 +464,15 @@ namespace dxvk {
     }
 
     // Deal with undefined shader inputs
-    uint32_t consumedInputs = shader->interfaceSlots().inputSlots;
+    uint32_t consumedInputs = shaderInfo.inputMask;
     uint32_t providedInputs = 0;
 
-    if (shader->stage() == VK_SHADER_STAGE_VERTEX_BIT) {
+    if (shaderInfo.stage == VK_SHADER_STAGE_VERTEX_BIT) {
       for (uint32_t i = 0; i < state.il.attributeCount(); i++)
         providedInputs |= 1u << state.ilAttributes[i].location();
-    } else if (shader->stage() != VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT) {
-      auto prevStage = getPrevStageShader(shader->stage());
-      providedInputs = prevStage->interfaceSlots().outputSlots;
+    } else if (shaderInfo.stage != VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT) {
+      auto prevStage = getPrevStageShader(shaderInfo.stage);
+      providedInputs = prevStage->info().outputMask;
     } else {
       // Technically not correct, but this
       // would need a lot of extra care
@@ -486,7 +510,8 @@ namespace dxvk {
 
 
   bool DxvkGraphicsPipeline::validatePipelineState(
-    const DxvkGraphicsPipelineStateInfo& state) const {
+    const DxvkGraphicsPipelineStateInfo&  state,
+          bool                            trusted) const {
     // Tessellation shaders and patches must be used together
     bool hasPatches = state.ia.primitiveTopology() == VK_PRIMITIVE_TOPOLOGY_PATCH_LIST;
 
@@ -504,8 +529,69 @@ namespace dxvk {
     if (state.il.attributeCount() > DxvkLimits::MaxNumVertexAttributes
      || state.il.bindingCount()   > DxvkLimits::MaxNumVertexBindings)
       return false;
-    
-    // No errors
+
+    // Exit here on the fast path, perform more thorough validation if
+    // the state vector comes from an untrusted source (i.e. the cache)
+    if (trusted)
+      return true;
+
+    // Validate shaders
+    if (!m_shaders.validate()) {
+      Logger::err("Invalid pipeline: Shader types do not match stage");
+      return false;
+    }
+
+    // Validate vertex input layout
+    const DxvkDevice* device = m_pipeMgr->m_device;
+    uint32_t ilLocationMask = 0;
+    uint32_t ilBindingMask = 0;
+
+    for (uint32_t i = 0; i < state.il.bindingCount(); i++)
+      ilBindingMask |= 1u << state.ilBindings[i].binding();
+
+    for (uint32_t i = 0; i < state.il.attributeCount(); i++) {
+      const DxvkIlAttribute& attribute = state.ilAttributes[i];
+
+      if (ilLocationMask & (1u << attribute.location())) {
+        Logger::err(str::format("Invalid pipeline: Vertex location ", attribute.location(), " defined twice"));
+        return false;
+      }
+
+      if (!(ilBindingMask & (1u << attribute.binding()))) {
+        Logger::err(str::format("Invalid pipeline: Vertex binding ", attribute.binding(), " not defined"));
+        return false;
+      }
+
+      VkFormatProperties formatInfo = device->adapter()->formatProperties(attribute.format());
+
+      if (!(formatInfo.bufferFeatures & VK_FORMAT_FEATURE_VERTEX_BUFFER_BIT)) {
+        Logger::err(str::format("Invalid pipeline: Format ", attribute.format(), " not supported for vertex buffers"));
+        return false;
+      }
+
+      ilLocationMask |= 1u << attribute.location();
+    }
+
+    // Validate rasterization state
+    if (state.rs.conservativeMode() != VK_CONSERVATIVE_RASTERIZATION_MODE_DISABLED_EXT) {
+      if (!device->extensions().extConservativeRasterization) {
+        Logger::err("Conservative rasterization not supported by device");
+        return false;
+      }
+
+      if (state.rs.conservativeMode() == VK_CONSERVATIVE_RASTERIZATION_MODE_UNDERESTIMATE_EXT
+       && !device->properties().extConservativeRasterization.primitiveUnderestimation) {
+        Logger::err("Primitive underestimation not supported by device");
+        return false;
+      }
+    }
+
+    // Validate depth-stencil state
+    if (state.ds.enableDepthBoundsTest() && !device->features().core.features.depthBounds) {
+      Logger::err("Depth bounds not supported by device");
+      return false;
+    }
+
     return true;
   }
   
