@@ -25,10 +25,18 @@ namespace dxvk {
      * \brief Adds a resource to track
      * \param [in] rc The resource to track
      */
+    template<DxvkAccess Access>
     void trackResource(Rc<DxvkResource>&& rc) {
-      rc->acquire();
-      m_resources.emplace_back(std::move(rc));
+      rc->acquire(Access);
+      m_resources.emplace_back(std::move(rc), Access);
     }
+
+    /**
+     * \brief Releases resources
+     *
+     * Marks all tracked resources as unused.
+     */
+    void notify();
     
     /**
      * \brief Resets the command list
@@ -40,7 +48,8 @@ namespace dxvk {
     
   private:
     
-    std::vector<Rc<DxvkResource>> m_resources;
+    std::vector<std::pair<Rc<DxvkResource>, DxvkAccess>> m_resources;
+    bool m_notified = false;
     
   };
   
