@@ -169,7 +169,8 @@ namespace dxvk {
     DxvkMemoryFlags mask(
       DxvkMemoryFlag::Small,
       DxvkMemoryFlag::GpuReadable,
-      DxvkMemoryFlag::GpuWritable);
+      DxvkMemoryFlag::GpuWritable,
+      DxvkMemoryFlag::Transient);
 
     if (hints.test(DxvkMemoryFlag::IgnoreConstraints))
       mask = DxvkMemoryFlags();
@@ -255,10 +256,10 @@ namespace dxvk {
       hints.clr(DxvkMemoryFlag::GpuWritable, DxvkMemoryFlag::GpuReadable);
     }
 
-    // Ignore all hints for host-visible allocations since they
+    // Ignore most hints for host-visible allocations since they
     // usually don't make much sense for those resources
     if (flags & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)
-      hints = DxvkMemoryFlags();
+      hints = hints & DxvkMemoryFlag::Transient;
 
     // Try to allocate from a memory type which supports the given flags exactly
     auto dedAllocPtr = dedAllocReq.prefersDedicatedAllocation ? &dedAllocInfo : nullptr;
@@ -506,7 +507,7 @@ namespace dxvk {
     // Try to waste a bit less system memory especially in
     // 32-bit applications due to address space constraints
     if (type.propertyFlags & VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT)
-      chunkSize = 16 << 20;
+      chunkSize = (env::is32BitHostPlatform() ? 16 : 64) << 20;
 
     // Reduce the chunk size on small heaps so
     // we can at least fit in 15 allocations
