@@ -8,6 +8,9 @@ namespace dxvk {
     DxvkOptions() { }
     DxvkOptions(const Config& config);
 
+    /// Enable debug utils (alternative to DXVK_PERF_EVENTS=1)
+    bool enableDebugUtils;
+
     /// Enable state cache
     bool enableStateCache;
 
