@@ -1,7 +1,6 @@
 #pragma once
 
 #include <atomic>
-#include <chrono>
 #include <condition_variable>
 #include <fstream>
 
@@ -9,6 +8,7 @@
 
 #include "../util/sha1/sha1_util.h"
 #include "../util/util_env.h"
+#include "../util/util_time.h"
 
 namespace dxvk {
   
@@ -22,22 +22,16 @@ namespace dxvk {
     
   public:
     
-    DxvkPipelineCache(const Rc<vk::DeviceFn>& vkd);
-    ~DxvkPipelineCache();
+    DxvkPipelineCache(const Rc<vk::DeviceFn>& vkd) { }
     
     /**
      * \brief Pipeline cache handle
      * \returns Pipeline cache handle
      */
     VkPipelineCache handle() const {
-      return m_handle;
+      return VK_NULL_HANDLE;
     }
     
-  private:
-    
-    Rc<vk::DeviceFn>        m_vkd;
-    VkPipelineCache         m_handle;
-    
   };
   
 }
