@@ -3,6 +3,8 @@
 #include "dxvk_resource.h"
 
 namespace dxvk {
+
+  class DxvkDevice;
   
   /**
    * \brief Sampler properties
@@ -36,6 +38,9 @@ namespace dxvk {
     
     /// Enables unnormalized coordinates
     VkBool32 usePixelCoord;
+
+    /// Enables non seamless cube map filtering
+    VkBool32 nonSeamless;
   };
   
   
@@ -51,7 +56,7 @@ namespace dxvk {
   public:
     
     DxvkSampler(
-      const Rc<vk::DeviceFn>&       vkd,
+            DxvkDevice*             device,
       const DxvkSamplerCreateInfo&  info);
     ~DxvkSampler();
     
@@ -68,7 +73,9 @@ namespace dxvk {
     Rc<vk::DeviceFn>      m_vkd;
     VkSampler             m_sampler = VK_NULL_HANDLE;
 
-    VkBorderColor getBorderColor(bool depthCompare, VkClearColorValue borderColor) const;
+    static VkBorderColor getBorderColor(
+      const Rc<DxvkDevice>&         device,
+      const DxvkSamplerCreateInfo&  info);
     
   };
   
