@@ -3,6 +3,7 @@
 #include <spirv/spirv.hpp>
 
 #include <iostream>
+#include <utility>
 #include <vector>
 
 #include "spirv_instruction.h"
@@ -208,9 +209,10 @@ namespace dxvk {
      * After this call, new instructions will be
      * appended to the stream. In other words,
      * this will restore default behaviour.
+     * \returns Previous instruction pointer
      */
-    void endInsertion() {
-      m_ptr = m_code.size();
+    size_t endInsertion() {
+      return std::exchange(m_ptr, m_code.size());
     }
     
   private:
@@ -220,4 +222,4 @@ namespace dxvk {
     
   };
   
-}
\ No newline at end of file
+}
