@@ -35,6 +35,7 @@ namespace dxvk {
      * multiple times                             */
     { R"(\\EvilWithin(Demo)?\.exe$)", {{
       { "d3d11.dcSingleUseMode",            "False" },
+      { "d3d11.cachedDynamicResources",     "vi"   },
     }} },
     /* Far Cry 3: Assumes clear(0.5) on an UNORM  *
      * format to result in 128 on AMD and 127 on  *
@@ -51,20 +52,28 @@ namespace dxvk {
      * after creating the DXGI swap chain         */
     { R"(\\Frostpunk\.exe$)", {{
       { "dxgi.deferSurfaceCreation",        "True" },
+      { "d3d11.cachedDynamicResources",     "c" },
     }} },
     /* Nioh: See Frostpunk, apparently?           */
     { R"(\\nioh\.exe$)", {{
       { "dxgi.deferSurfaceCreation",        "True" },
     }} },
     /* Quantum Break: Mever initializes shared    *
-     * memory in one of its compute shaders       */
+     * memory in one of its compute shaders.      *
+     * Also loves using MAP_WRITE on the same     *
+     * set of resources multiple times per frame. */
     { R"(\\QuantumBreak\.exe$)", {{
       { "d3d11.zeroInitWorkgroupMemory",    "True" },
+      { "d3d11.maxImplicitDiscardSize",     "-1"   },
     }} },
     /* Anno 2205: Random crashes with state cache */
     { R"(\\anno2205\.exe$)", {{
       { "dxvk.enableStateCache",            "False" },
     }} },
+    /* Anno 1800: Poor performance without this   */
+    { R"(\\Anno1800\.exe$)", {{
+      { "d3d11.cachedDynamicResources",     "c"    },
+    }} },
     /* Fifa '19+: Binds typed buffer SRV to shader *
      * that expects raw/structured buffer SRV     */
     { R"(\\FIFA(19|[2-9][0-9])(_demo)?\.exe$)", {{
@@ -112,8 +121,9 @@ namespace dxvk {
     }} },
     /* NieR Replicant                             */
     { R"(\\NieR Replicant ver\.1\.22474487139\.exe)", {{
-      { "dxgi.syncInterval",                "1"   },
-      { "dxgi.maxFrameRate",                "60"  },
+      { "dxgi.syncInterval",                "1"    },
+      { "dxgi.maxFrameRate",                "60"   },
+      { "d3d11.cachedDynamicResources",     "vi"   },
     }} },
     /* SteamVR performance test                   */
     { R"(\\vr\.exe$)", {{
@@ -144,13 +154,13 @@ namespace dxvk {
      * game parts                                   */
     { R"(\\Crysis3\.exe$)", {{
       { "dxgi.customVendorId",              "10de" },
-      { "d3d11.apitraceMode",               "True" },
+      { "d3d11.cachedDynamicResources",     "a"    },
     }} },
     /* Crysis 3 Remastered                          *
      * Apitrace mode helps massively in cpu bound   *
      * game parts                                   */
     { R"(\\Crysis3Remastered\.exe$)", {{
-      { "d3d11.apitraceMode",               "True" },
+      { "d3d11.cachedDynamicResources",     "a"    },
     }} },
     /* Atelier series - games try to render video *
      * with a D3D9 swap chain over the DXGI swap  *
@@ -158,12 +168,16 @@ namespace dxvk {
     { R"(\\Atelier_(Ayesha|Escha_and_Logy|Shallie)(_EN)?\.exe$)", {{
       { "d3d9.deferSurfaceCreation",        "True" },
     }} },
+    /* Atelier Firis                              */
+    { R"(\\A18\.exe$)", {{
+      { "d3d9.deferSurfaceCreation",        "True" },
+    }} },
     /* Atelier Rorona/Totori/Meruru               */
     { R"(\\A(11R|12V|13V)_x64_Release(_en)?\.exe$)", {{
       { "d3d9.deferSurfaceCreation",        "True" },
     }} },
     /* Just how many of these games are there?    */
-    { R"(\\Atelier_(Lulua|Lydie_and_Suelle|Ryza(_2)?)\.exe$)", {{
+    { R"(\\Atelier_(Lulua|Lydie_and_Suelle|Ryza(_2)?|Sophie_2)\.exe$)", {{
       { "d3d9.deferSurfaceCreation",        "True" },
     }} },
     /* ...                                        */
@@ -198,23 +212,27 @@ namespace dxvk {
     /* Darksiders Warmastered - apparently reads  *
      * from write-only mapped buffers             */
     { R"(\\darksiders1\.exe$)", {{
-      { "d3d11.apitraceMode",               "True" },
+      { "d3d11.cachedDynamicResources",     "a"    },
     }} },
     /* Monster Hunter World                       */
     { R"(\\MonsterHunterWorld\.exe$)", {{
-      { "d3d11.apitraceMode",               "True" },
+      { "d3d11.cachedDynamicResources",     "a"    },
     }} },
     /* Kingdome Come: Deliverance                 */
     { R"(\\KingdomCome\.exe$)", {{
-      { "d3d11.apitraceMode",               "True" },
+      { "d3d11.cachedDynamicResources",     "a"    },
     }} },
     /* Homefront: The Revolution                  */
     { R"(\\Homefront2_Release\.exe$)", {{
-      { "d3d11.apitraceMode",               "True" },
+      { "d3d11.cachedDynamicResources",     "a"    },
     }} },
     /* Sniper Ghost Warrior Contracts             */
     { R"(\\SGWContracts\.exe$)", {{
-      { "d3d11.apitraceMode",               "True" },
+      { "d3d11.cachedDynamicResources",     "a"    },
+    }} },
+    /* Armored Warfare             */
+    { R"(\\armoredwarfare\.exe$)", {{
+      { "d3d11.cachedDynamicResources",     "c"    },
     }} },
     /* Shadow of the Tomb Raider - invariant      *
      * position breaks character rendering on NV  */
@@ -244,6 +262,7 @@ namespace dxvk {
     /* Final Fantasy XIV - Stuttering on NV       */
     { R"(\\ffxiv_dx11\.exe$)", {{
       { "dxvk.shrinkNvidiaHvvHeap",         "True" },
+      { "d3d11.cachedDynamicResources",     "vi"   },
     }} },
     /* God of War - relies on NVAPI/AMDAGS for    *
      * barrier stuff, needs nvapi for DLSS        */
@@ -251,6 +270,46 @@ namespace dxvk {
       { "d3d11.ignoreGraphicsBarriers",     "True" },
       { "d3d11.relaxedBarriers",            "True" },
       { "dxgi.nvapiHack",                   "False" },
+      { "dxgi.maxFrameLatency",             "1"    },
+    }} },
+    /* AoE 2 DE - runs poorly for some users      */
+    { R"(\\AoE2DE_s\.exe$)", {{
+      { "d3d11.cachedDynamicResources",     "a"    },
+    }} },
+    /* Total War: Warhammer III                   */
+    { R"(\\Warhammer3\.exe$)", {{
+      { "d3d11.maxDynamicImageBufferSize",  "4096" },
+    }} },
+    /* Assassin's Creed 3 and 4                   */
+    { R"(\\ac(3|4bf)[sm]p\.exe$)", {{
+      { "d3d11.cachedDynamicResources",     "a"    },
+    }} },
+    /* Stranger of Paradise - FF Origin           */
+    { R"(\\SOPFFO\.exe$)", {{
+      { "d3d9.deferSurfaceCreation",        "True" },
+    /* Small Radios Big Televisions               */
+    }} },
+    { R"(\\SRBT\.exe$)", {{
+      { "d3d9.deferSurfaceCreation",        "True" },
+    }} },
+    /* A Way Out: fix for stuttering and low fps  */
+    { R"(\\AWayOut(_friend)?\.exe$)", {{
+      { "dxgi.maxFrameLatency",                "1" },
+    }} },
+    /* Garden Warfare 2
+       Won't start on amd Id without atiadlxx     */
+    { R"(\\GW2.Main_Win64_Retail\.exe$)", {{
+      { "dxgi.customVendorId",           "10de"   },
+    }} },
+    /* DayZ */
+    { R"(\\DayZ_x64\.exe$)", {{
+      { "d3d11.cachedDynamicResources",     "cr" },
+    }} },
+    /* Stray - writes to the same UAV every draw, *
+     * presumably for culling, which doesn't play *
+     * nicely with D3D11 without vendor libraries */
+    { R"(\\Stray-Win64-Shipping\.exe$)", {{
+      { "d3d11.ignoreGraphicsBarriers",     "True" },
     }} },
 
     /**********************************************/
@@ -279,13 +338,9 @@ namespace dxvk {
     { R"(\\Gothic(3|3Final| III Forsaken Gods)\.exe$)", {{
       { "d3d9.supportDFFormats",            "False" },
     }} },
-    /* Risen                                      */
-    { R"(\\Risen[23]?\.exe$)", {{
-      { "d3d9.invariantPosition",           "True" },
-    }} },
     /* Sonic Adventure 2                          */
     { R"(\\Sonic Adventure 2\\(launcher|sonic2app)\.exe$)", {{
-      { "d3d9.floatEmulation",              "False" },
+      { "d3d9.floatEmulation",              "Strict" },
     }} },
     /* The Sims 2,
        Body Shop,
@@ -302,17 +357,16 @@ namespace dxvk {
       { "d3d9.memoryTrackTest",             "True" },
     }} },
     /* Dead Space uses the a NULL render target instead
-       of a 1x1 one if DF24 is NOT supported      */
+       of a 1x1 one if DF24 is NOT supported     
+       Mouse and physics issues above 60 FPS
+       Built-in Vsync Locks the game to 30 FPS    */
     { R"(\\Dead Space\.exe$)", {{
       { "d3d9.supportDFFormats",                 "False" },
-    }} },
-    /* Halo 2                                     */
-    { R"(\\halo2\.exe$)", {{
-      { "d3d9.invariantPosition",           "True" },
+      { "d3d9.maxFrameRate",                     "60" },
+      { "d3d9.presentInterval",                  "1" },
     }} },
     /* Halo CE/HaloPC                             */
     { R"(\\halo(ce)?\.exe$)", {{
-      { "d3d9.invariantPosition",           "True" },
       // Game enables minor decal layering fixes
       // specifically when it detects AMD.
       // Avoids chip being detected as unsupported
@@ -385,7 +439,6 @@ namespace dxvk {
     /* Battlefield 2 (bad z-pass)                 */
     { R"(\\BF2\.exe$)", {{
       { "d3d9.longMad",                     "True" },
-      { "d3d9.invariantPosition",           "True" },
     }} },
     /* SpellForce 2 Series                        */
     { R"(\\SpellForce2.*\.exe$)", {{
@@ -486,9 +539,87 @@ namespace dxvk {
     { R"(\\BBCF\.exe$)", {{
       { "d3d9.floatEmulation",              "Strict" },
     }} },
-    /* James Cameron's Avatar needs invariantPosition to fix black flickering vegetation */
-    { R"(\\Avatar\.exe$)", {{
-      { "d3d9.invariantPosition",              "True" },
+    /* Resident Evil games                      */
+    { R"(\\(rerev|rerev2|re0hd|bhd|re5dx9|BH6)\.exe$)", {{
+      { "d3d9.allowDirectBufferMapping",                "False" },
+    }} },
+    /* Limbo                                    */
+    { R"(\\limbo\.exe$)", {{
+      { "d3d9.maxFrameRate",                "60" },
+    }} },
+    /* Warhammer: Return of Reckoning Launcher
+       Forcing SM1 fixes a black window otherwise caused by
+       the lack of support for partial presentation */
+    { R"(\\RoRLauncher\.exe$)", {{
+      { "d3d9.shaderModel",                 "1" },
+    }} },
+    /* Halo CE SPV3 launcher
+       Same issue as Warhammer: RoR above       */
+    { R"(\\spv3\.exe$)", {{
+      { "d3d9.shaderModel",                 "1" },
+    }} },
+    /* Star Wars The Force Unleashed 2          *
+     * Black particles because it tries to bind *
+     * a 2D texture for a shader that           *
+     * declares a 3d texture.                   */
+    { R"(\\SWTFU2\.exe$)", {{
+      { "d3d9.forceSamplerTypeSpecConstants",  "True" },
+    }} },
+    /* Scrapland (Remastered)                   */
+    { R"(\\Scrap\.exe$)", {{
+      { "d3d9.deferSurfaceCreation",        "True" },
+    }} },
+    /* Majesty 2 (Collection)                   *
+     * Crashes on UMA without a memory limit,   *
+     * since the game(s) will allocate all      *
+     * available VRAM on startup.               */
+    { R"(\\Majesty2\.exe$)", {{
+      { "d3d9.memoryTrackTest",             "True" },
+      { "d3d9.maxAvailableMemory",          "2048" },
+    }} },
+    /* Myst V End of Ages                             
+       Game has white textures on amd radv.
+       Expects Nvidia, Intel or ATI VendorId.
+       "Radeon" in gpu description also works   */
+    { R"(\\eoa\.exe$)", {{
+      { "d3d9.customVendorId",              "10de" },
+    }} },
+    /* Beyond Good And Evil                     *
+     * Fixes missing sun and light shafts       */
+    { R"(\\BGE\.exe$)", {{
+      { "d3d9.allowDoNotWait",              "False" },
+    }} },
+    /* Supreme Commander & Forged Alliance Forever */
+    { R"(\\(SupremeCommander|ForgedAlliance)\.exe$)", {{
+      { "d3d9.floatEmulation",            "Strict" },
+    }} },
+    /* Star Wars The Old Republic */
+    { R"(\\swtor\.exe$)", {{
+      { "d3d9.forceSamplerTypeSpecConstants", "True" },
+    }} },
+    /* Bionic Commando                          
+       Physics break at high fps               */
+    { R"(\\bionic_commando\.exe$)", {{
+      { "d3d9.maxFrameRate",                "60" },
+    }} },
+    /* Port Royale 3                            *
+     * Fixes infinite loading screens           */
+    { R"(\\PortRoyale3\.exe$)", {{
+      { "d3d9.allowDoNotWait",           "False" },
+    }} },
+    /* Need For Speed 3 modern patch            */
+    { R"(\\nfs3\.exe$)", {{
+      { "d3d9.enableDialogMode",          "True" },
+    }} },
+    /* Ninja Blade                              *
+     * Transparent main character on Nvidia     */
+    { R"(\\NinjaBlade\.exe$)", {{
+      { "d3d9.alphaTestWiggleRoom",       "True" },
+    }} },
+    /* YS Origin                                *
+     * Helps very bad frametimes in some areas  */
+    { R"(\\yso_win\.exe$)", {{
+      { "d3d9.maxFrameLatency",              "1" },
     }} },
   }};
 
@@ -689,6 +820,10 @@ namespace dxvk {
     if (appConfig != g_appDefaults.end()) {
       // Inform the user that we loaded a default config
       Logger::info(str::format("Found built-in config:"));
+
+      for (auto& pair : appConfig->second.m_options)
+        Logger::info(str::format("  ", pair.first, " = ", pair.second));
+
       return appConfig->second;
     }
 
