@@ -4,6 +4,7 @@
 #include <condition_variable>
 #include <functional>
 #include <mutex>
+#include <thread>
 
 #include "util_error.h"
 
@@ -14,6 +15,7 @@
 
 namespace dxvk {
 
+#ifdef _WIN32
   /**
    * \brief Thread priority
    */
@@ -147,6 +149,10 @@ namespace dxvk {
     inline void yield() {
       SwitchToThread();
     }
+
+    inline uint32_t get_id() {
+      return uint32_t(GetCurrentThreadId());
+    }
   }
 
 
@@ -323,4 +329,19 @@ namespace dxvk {
 
   };
 
+#else
+  using mutex              = std::mutex;
+  using thread             = std::thread;
+  using recursive_mutex    = std::recursive_mutex;
+  using condition_variable = std::condition_variable;
+
+  namespace this_thread {
+    inline void yield() {
+      std::this_thread::yield();
+    }
+
+    uint32_t get_id();
+  }
+#endif
+
 }
