@@ -1,5 +1,6 @@
 #pragma once
 
+#include <cstring>
 #include <string>
 #include <sstream>
 #include <vector>
@@ -16,6 +17,8 @@ namespace dxvk::str {
   void tows(const char* mbs, WCHAR (&wcs)[N]) {
     return tows(mbs, wcs, N);
   }
+
+  std::wstring tows(const char* mbs);
   
   inline void format1(std::stringstream&) { }
 
@@ -37,5 +40,11 @@ namespace dxvk::str {
     format1(stream, args...);
     return stream.str();
   }
+
+  inline void strlcpy(char* dst, const char* src, size_t count) {
+    std::strncpy(dst, src, count);
+    if (count > 0)
+      dst[count - 1] = '\0';
+  }
   
 }
